package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.client.bean.AdverseReaction;
import com.diagbot.client.bean.Medicition;
import com.diagbot.client.bean.MedicitionClass;
import com.diagbot.dto.ConceptRes;
import com.diagbot.entity.Concept;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.enums.DisTypeEnum;
import com.diagbot.enums.LexiconRSTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:治疗方案业务处理
 * @Author:zhaops
 * @time: 2019/5/8 17:16
 */
@Component
public class TreatmentFacade {

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    ConceptDetailFacade conceptDetailFacade;
    @Autowired
    DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    /**
     * 获取治疗方案
     *
     * @param
     * @return
     */
    public Map<String, Object> getTreatment(Map<String, JSONObject> treatmentMap, String diseaseName, Integer disType) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        List<String> conceptNameList = Lists.newLinkedList();
        conceptNameList.add(diseaseName);
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        conceptBaseVO.setName(diseaseName);
        conceptBaseVO.setLibType(LexiconTypeEnum.DIAGNOSIS.getKey());
        Concept disease = conceptFacade.exist(conceptBaseVO);
        if (disease == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "诊断不存在");
        }
        if (treatmentMap == null || treatmentMap.size() == 0 || !treatmentMap.containsKey(diseaseName)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "未找到治疗方案");
        }

        //治疗方案对应诊断
        retMap.put("diseaseName", diseaseName);

        //获取知识图谱治疗方案
        JSONObject treatmentJson = treatmentMap.get(diseaseName);
        if (treatmentJson == null || treatmentJson.isEmpty()) {
            return retMap;
        }
        //推荐药物
        List<MedicitionClass> drugsList
                = FastJsonUtils.getJsonToListByKey(treatmentJson.toString(), "treatment", MedicitionClass.class);
        //不良反应
        List<AdverseReaction> adverseReactionList
                = FastJsonUtils.getJsonToListByKey(treatmentJson.toString(), "adverseEvent", AdverseReaction.class);
        if (ListUtil.isNotEmpty(drugsList)) {
            //药品大类
            List<String> cateBigNameList = drugsList
                    .stream()
                    .map(drugs -> drugs.getBigdrugsName())
                    .collect(Collectors.toList());
            conceptNameList.addAll(cateBigNameList);
            //药品小类
            List<String> cateSmallNameList = drugsList
                    .stream()
                    .map(drugs -> drugs.getSubdrugsName())
                    .collect(Collectors.toList());
            conceptNameList.addAll(cateSmallNameList);
            //药品
            List<Medicition> medicitionTotalList = Lists.newLinkedList();
            for (MedicitionClass medicitionClass : drugsList) {
                if (medicitionClass.getMedicitionsList() != null) {
                    medicitionTotalList.addAll(medicitionClass.getMedicitionsList());
                }
            }
            List<String> drugNameList = medicitionTotalList
                    .stream()
                    .map(m -> m.getMedicitionName())
                    .collect(Collectors.toList());
            conceptNameList.addAll(drugNameList);
        }
        if (ListUtil.isNotEmpty(adverseReactionList)) {
            List<String> adNameList = adverseReactionList
                    .stream()
                    .map(a -> a.getName())
                    .collect(Collectors.toList());
            conceptNameList.addAll(adNameList);
        }
        //概念
        List<Concept> conceptList = conceptFacade.getListByNames(conceptNameList);
        //概念map
        Map<String, Map<Long, Concept>> conceptMap = conceptList2Map(conceptList);
        //说明信息是否存在
        Map<String, Map<Long, List<ConceptDetail>>> hasConDetailMap = conceptDetailFacade.hasConDetail(conceptList);

        //判断是否慢病
        Boolean isChronic = false;
        if (disType != null && disType.equals(DisTypeEnum.CHRONIC.getKey())) {
            //验证是否慢病
            conceptBaseVO.setName("慢病");
            conceptBaseVO.setLibType(LexiconTypeEnum.TYPES_OF_DISEASE.getKey());
            Concept chronicConcept = conceptFacade.exist(conceptBaseVO);
            ConceptWrapper conceptWrapper = new ConceptWrapper();
            conceptWrapper.setStartId(disease.getId());
            conceptWrapper.setStartName(diseaseName);
            conceptWrapper.setStartType(LexiconTypeEnum.DIAGNOSIS.getKey());
            conceptWrapper.setEndId(chronicConcept.getId());
            conceptWrapper.setEndName(chronicConcept.getLibName());
            conceptWrapper.setEndType(LexiconTypeEnum.TYPES_OF_DISEASE.getKey());
            conceptWrapper.setRelationType(LexiconRSTypeEnum.INCLUDE_OF.getKey());
            List<ConceptRes> conceptResList = conceptFacade.getConcept(conceptWrapper);
            isChronic = ListUtil.isNotEmpty(conceptResList);
        }

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("treatmentMap", retMap);
            invokeParams.put("diseaseName", diseaseName);
            invokeParams.put("drugsList", drugsList);
            invokeParams.put("conceptMap", conceptMap);
            invokeParams.put("hasConDetailMap", hasConDetailMap);
            invokeParams.put("adverseReactionList", adverseReactionList);
            invokeParams.put("isChronic", isChronic);

            retMap
                    = dataBeanAggregateQueryFacade.get("setTreatmentAll", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        //判断是否慢病，慢病增加回访时间，上次用药，不良反应
        if (isChronic) {
            if (retMap == null) {
                retMap = new LinkedHashMap<>();
            }
            //慢病，增加回访时间，暂时前端写死， 以后由只是图谱返回
            retMap.put("followUp", null);
            //上次用药-icss层处理
            retMap.put("drugHistory", null);
        }

        //        //一般治疗&&手术治疗
        //        if (hasConDetailMap != null && hasConDetailMap.get(diseaseName) != null
        //                && ListUtil.isNotEmpty(hasConDetailMap
        //                .get(diseaseName)
        //                .get(Long.valueOf(LexiconTypeEnum.DIAGNOSIS.getKey())))) {
        //            List<ConceptDetail> diseaseConDetailList
        //                    = hasConDetailMap.get(diseaseName).get(Long.valueOf(LexiconTypeEnum.DIAGNOSIS.getKey()));
        //            for (ConceptDetail conceptDetail : diseaseConDetailList) {
        //                ConceptDetailDTO conceptDetailDTO = new ConceptDetailDTO();
        //                List<String> positionList = Arrays.asList(conceptDetail.getPosition().split(",|，"));
        //                if (positionList.contains(String.valueOf(PositionTypeEnum.T3.getKey()))
        //                        && retMap.get("commonTreatment") == null) {
        //                    BeanUtil.copyProperties(conceptDetail, conceptDetailDTO);
        //                    retMap.put("commonTreatment", conceptDetailDTO);
        //                }
        //                if (positionList.contains(String.valueOf(PositionTypeEnum.T4.getKey()))
        //                        && retMap.get("surgeryTreatment") == null) {
        //                    BeanUtil.copyProperties(conceptDetail, conceptDetailDTO);
        //                    retMap.put("surgeryTreatment", conceptDetailDTO);
        //                }
        //            }
        //        }
        //        if (!retMap.containsKey("commonTreatment")) {
        //            retMap.put("commonTreatment", null);
        //        }
        //        if (!retMap.containsKey("surgeryTreatment")) {
        //            retMap.put("surgeryTreatment", null);
        //        }

        //        if (ListUtil.isNotEmpty(drugsList)) {
        //            for (MedicitionClass medicitionClass : drugsList) {
        //                if (conceptMap != null
        //                        && conceptMap.get(medicitionClass.getBigdrugsName()) != null
        //                        && conceptMap.get(medicitionClass
        //                        .getBigdrugsName())
        //                        .containsKey(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey()))) {
        //                    Concept cateBigConcept = conceptMap
        //                            .get(medicitionClass.getBigdrugsName())
        //                            .get(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey()));
        //                    if (cateBigConcept != null) {
        //                        medicitionClass.setBigdrugsConceptId(cateBigConcept.getId());
        //                        if (hasConDetailMap != null
        //                                && hasConDetailMap.get(cateBigConcept.getLibName()) != null
        //                                && hasConDetailMap.get(cateBigConcept.getLibName())
        //                                .containsKey(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey()))) {
        //                            medicitionClass.setShowInfo("1");
        //                        } else {
        //                            medicitionClass.setShowInfo("0");
        //                        }
        //                    }
        //                }
        //                medicitionClass.setBigdrgusLibType(LexiconTypeEnum.DRUG_CATEGORY_BIG.getKey());
        //                medicitionClass.setBigdrugsType(ConceptTypeEnum.Drug_Category_Big.getKey());
        //                if (conceptMap != null
        //                        && conceptMap.get(medicitionClass.getSubdrugsName()) != null
        //                        && conceptMap.get(medicitionClass.getSubdrugsName())
        //                        .containsKey(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_SMALL.getKey()))) {
        //                    Concept cateSmallConcept = conceptMap
        //                            .get(medicitionClass.getSubdrugsName())
        //                            .get(Long.valueOf(LexiconTypeEnum.DRUG_CATEGORY_SMALL.getKey()));
        //                    if (cateSmallConcept != null) {
        //                        medicitionClass.setSubdrugsConceptId(cateSmallConcept.getId());
        //                    }
        //                }
        //                medicitionClass.setSubdrugsLibType(LexiconTypeEnum.DRUG_CATEGORY_SMALL.getKey());
        //                medicitionClass.setSubdrugsType(ConceptTypeEnum.Drug_Category_Small.getKey());
        //
        //                LinkedList<Medicition> medicitionList = medicitionClass.getMedicitionsList();
        //                for (Medicition medicition : medicitionList) {
        //                    if (conceptMap != null
        //                            && conceptMap.get(medicition.getMedicitionName()) != null
        //                            && conceptMap.get(medicition.getMedicitionName())
        //                            .containsKey(Long.valueOf(LexiconTypeEnum.DRUGS.getKey()))) {
        //                        Concept drugConcept = conceptMap
        //                                .get(medicition.getMedicitionName())
        //                                .get(Long.valueOf(LexiconTypeEnum.DRUGS.getKey()));
        //                        if (drugConcept != null) {
        //                            medicition.setConceptId(drugConcept.getId());
        //                        }
        //                    }
        //                    medicition.setLibType(LexiconTypeEnum.DRUGS.getKey());
        //                    medicition.setType(ConceptTypeEnum.Drug.getKey());
        //                    if (hasConDetailMap != null
        //                            && hasConDetailMap.get(medicition.getMedicitionName()) != null
        //                            && hasConDetailMap.get(medicition.getMedicitionName())
        //                            .containsKey(Long.valueOf(LexiconTypeEnum.DRUGS.getKey()))) {
        //                        medicition.setShowInfo("1");
        //                    } else {
        //                        medicition.setShowInfo("0");
        //                    }
        //                }
        //                medicitionClass.setMedicitionsList(medicitionList);
        //            }
        //        }
        //        retMap.put("treatment", drugsList);


        //        //不良反应
        //        for (AdverseReaction adverseReaction : adverseReactionList) {
        //            if (conceptMap != null
        //                    && conceptMap.get(adverseReaction.getName()) != null
        //                    && conceptMap.get(adverseReaction.getName())
        //                    .containsKey(Long.valueOf(LexiconTypeEnum.SIDE_EFFECTS.getKey()))) {
        //                Concept adConcept = conceptMap
        //                        .get(adverseReaction.getName())
        //                        .get(Long.valueOf(LexiconTypeEnum.SIDE_EFFECTS.getKey()));
        //                if (adConcept != null) {
        //                    adverseReaction.setConceptId(adConcept.getId());
        //                }
        //            }
        //            adverseReaction.setLibType(LexiconTypeEnum.SIDE_EFFECTS.getKey());
        //            adverseReaction.setType(ConceptTypeEnum.SIDE_EFFECTS.getKey());
        //            if (hasConDetailMap != null
        //                    && hasConDetailMap.get(adverseReaction.getName()) != null
        //                    && hasConDetailMap.get(adverseReaction.getName())
        //                    .containsKey(Long.valueOf(LexiconTypeEnum.SIDE_EFFECTS.getKey()))) {
        //                adverseReaction.setShowInfo("1");
        //            } else {
        //                adverseReaction.setShowInfo("0");
        //            }
        //        }
        //        retMap.put("adverseReactions", adverseReactionList);


        return retMap;
    }

    /**
     * 转换概念list到map  Map<概念名称,Map<词性,概念对象>>
     *
     * @param conceptList
     * @return
     */
    public Map<String, Map<Long, Concept>> conceptList2Map(List<Concept> conceptList) {
        Map<String, Map<Long, Concept>> retMap = new LinkedHashMap<>();
        Map<Long, Concept> retSubMap = new LinkedHashMap<>();
        for (Concept concept : conceptList) {
            if (retMap.containsKey(concept.getLibName())) {
                retSubMap = retMap.get(concept.getLibName());
            }
            if (retSubMap == null) {
                retSubMap = new LinkedHashMap<>();
            }
            retSubMap.put(concept.getLibType(), concept);
            retMap.put(concept.getLibName(), retSubMap);
            retSubMap = new LinkedHashMap<>();
        }
        return retMap;
    }
}